uniform sampler2D u_texture0;
uniform sampler2D u_textureSrc;

precision mediump float;

uniform int uApplyMask;
uniform int uMaskType;
uniform vec2 uMaskSize;
uniform vec2 uMaskCenter;
uniform vec2 uMaskSqr;
uniform vec2 uGradientDepth;
uniform int uInverseMask;

varying vec2 blurCoordinates[9];

void main(void)
{
	lowp vec4 sum = vec4(0.0);
	sum += texture2D(u_texture0, blurCoordinates[0]) * 0.16;
	sum += texture2D(u_texture0, blurCoordinates[1]) * 0.15;
	sum += texture2D(u_texture0, blurCoordinates[2]) * 0.15;
	sum += texture2D(u_texture0, blurCoordinates[3]) * 0.125;
	sum += texture2D(u_texture0, blurCoordinates[4]) * 0.125;
	sum += texture2D(u_texture0, blurCoordinates[5]) * 0.095;
	sum += texture2D(u_texture0, blurCoordinates[6]) * 0.095;
	sum += texture2D(u_texture0, blurCoordinates[7]) * 0.05;
	sum += texture2D(u_texture0, blurCoordinates[8]) * 0.05;

	if(uApplyMask > 0)
	{
        if(uMaskType == 0) // Box
        {
            vec2 v2Dis = abs(blurCoordinates[0] - uMaskCenter);
            vec2 v2Ratio = clamp((uMaskSize - v2Dis) / uGradientDepth, 0.0, 1.0);
            float fRatio = min(v2Ratio.x, v2Ratio.y);
            if(uInverseMask > 0)
                sum = mix(sum, texture2D(u_textureSrc, blurCoordinates[0]), fRatio);
            else
                sum = mix(texture2D(u_textureSrc, blurCoordinates[0]), sum, fRatio);
        }
        else //if(uMaskType == 1) // Circle
        {
            vec2 v2Dis = blurCoordinates[0] - uMaskCenter;
            v2Dis = v2Dis * v2Dis / uMaskSqr;
            float fRatio = (1.0 - (v2Dis.x + v2Dis.y)) / uGradientDepth.x;
            fRatio = clamp(fRatio, 0.0, 1.0);
            if(uInverseMask > 0)
                sum = mix(sum, texture2D(u_textureSrc, blurCoordinates[0]), fRatio);
            else
                sum = mix(texture2D(u_textureSrc, blurCoordinates[0]), sum, fRatio);
        }
	}

	gl_FragColor = sum;
}